#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <dirent.h>
#include "superbird.h"

#include "img_folder.h"
#include "img_file.h"
#include "img_restart.h"
#include "img_power.h"

#define BTN_UP 0x01
#define BTN_DN 0x02
#define BTN_PUSH 0x00

char *color_sel = "\033[44;37m\033s";
char *color_unsel = "\033[47;30m\033s";
char *lcd_clear = "\033c\033s";

int lcd_txt;
int lcd_gfx;
int superbird;

unsigned char get_button() {
	unsigned char status;
	unsigned char temp = 0x03;
	for(;;) {
		status = ioctl(superbird, SUPERBIRD_GET_STATUS) & 0x03;
		if(status != 0x03) {
			if(temp != 0x03) return temp;
			else temp = status;
		}
		usleep(10000);
	}
}

void button_wait_unpushed() {
	unsigned char status = 0x00;
	while(status != 0x03) {
		status = ioctl(superbird, SUPERBIRD_GET_STATUS) & 0x03;
		usleep(10000);
	}
}

inline void lcd_twrite(const char *s) {
	write(lcd_txt, s, strlen(s));
}

int show_menu(char **items, int nitems, int init, unsigned char **icons) {
	int start = 0;
	int sel = init;
	int i;
	char buf[32];
	unsigned char btn;
	
	if(sel > 7) start = sel-7;
	
	for(;;) {
		lcd_twrite(color_unsel);
		lcd_twrite(lcd_clear);
		for(i=start; i<start + 8; i++) {
			if(i == nitems) break;
			if(i == sel) lcd_twrite(color_sel);
			else lcd_twrite(color_unsel);
			sprintf(buf, "\033s  % -15s", items[i]);
			lcd_twrite(buf);
			if(i<start+7) lcd_twrite("\033s\n");
			if(icons[i] != NULL) {
				icons[i][0] = 3;
				icons[i][1] = 1;
				icons[i][2] = (i-start) * 14 + 2;
				write(lcd_gfx, icons[i], 12*12*3 + 5);
			}
		}
		lcd_twrite("");
		button_wait_unpushed();
		btn = get_button();
		switch(btn) {
			case BTN_PUSH: 
				return sel;
			case BTN_DN:
				if(sel < nitems-1) sel++;
				if(sel > start + 7) start++;
				break;
			case BTN_UP:
				if(sel > 0) sel--;
				if(sel < start) start--;
				break;
		}
		
	}	
}

void dir_menu(const char *path, const char *title) {
	char *items[256];
	char buf[256];
	int nitems = 1;
	int sel = 0;
	DIR *progdir;
	struct dirent *ent;
	unsigned char types[256];
	int i;
	char *args[2] = {NULL, NULL};
	unsigned char *icons[256];
	icons[0] = NULL;
	
	items[0] = "Back";
	
	progdir = opendir(path);
	if(progdir == NULL) {
		perror("Can't open directory");
		exit(-1);
	}
	while(ent = readdir(progdir)) {
		if(!strcmp(ent->d_name, ".")) continue;
		if(!strcmp(ent->d_name, "..")) continue;
		items[nitems] = malloc(16);
		types[nitems] = ent->d_type;
		if(ent->d_type == DT_DIR) icons[nitems] = img_folder;
		else icons[nitems] = img_file;
 		strncpy(items[nitems], ent->d_name, 15);
		nitems++;
		if(nitems == 256) break;
	}
	closedir(progdir);
	
	for(;;) {
		snprintf(buf, 256, "\033]2;%s\07", title);
		lcd_twrite(buf);
		sel = show_menu(items, nitems, sel, icons);
		if(sel == 0) {
			break;
		} else {
			strcpy(buf, path);
			strcat(buf, "/");
			strcat(buf, items[sel]);
			if(types[sel] == DT_DIR) {
				dir_menu(buf, items[sel]);
			} else {
				button_wait_unpushed();
				args[0] = buf;
				execv(buf, args);
			}
		}
	}
	
	for(i=1; i<nitems; i++) {
		free(items[i]);
	}
}

void main_menu() {
	int sel;
	char *restart_args[4] = {"shutdown", "-r", "now", NULL};
	lcd_twrite("\033]2;Main Menu\07");
	char *items[4] = {"Programs", "Settings", "Restart", "Shut Down"};
	unsigned char *icons[4] = {img_folder, img_folder, img_restart, img_power};
	sel = show_menu(items, 4, 0, icons);
	switch(sel) {
		case 0:
			dir_menu("/root/programs", "Programs");
			break;
		case 1:
			dir_menu("/root/settings", "Settings");
			break;
		case 2:
			lcd_twrite("\033]2;Restarting\07\033[37;40m\33cPlease wait...");
			execvp("shutdown", restart_args);
			break;
		case 3:
			ioctl(superbird, SUPERBIRD_POWER_DOWN);
			break;
	}
}

int main(int argc, char **argv) {
	lcd_txt = open("/dev/lcd-txt", O_WRONLY);
	if(lcd_txt < 0) {
		perror("opening /dev/lcd-txt");
		exit(-1);
	}
	
	lcd_gfx = open("/dev/lcd-gfx", O_WRONLY);
	if(lcd_txt < 0) {
		perror("opening /dev/lcd-gfx");
		exit(-1);
	}
	
	superbird = open("/dev/superbird", O_WRONLY);
	if(lcd_txt < 0) {
		perror("opening /dev/superbird");
		exit(-1);
	}
	
	for(;;) {
		main_menu();
	}
}
